﻿/**
* 直線
* ax + by + c = 0
* @author nutsu
* @version 0.2
*/

package {
	
	import flash.geom.Point;

	public class Line extends Object{
		
		public var a:Number;
		public var b:Number;
		public var c:Number;
		
		/**
		* Constructor
		* @param	a_
		* @param	b_
		* @param	c_
		*/
		public function Line( a_:Number, b_:Number, c_:Number ){
			a = a_;
			b = b_;
			c = c_;
		}
		
		/**
		* 任意の点を通る直線
		* @param	通る点
		* @param	ベクトル
		* @return	Line
		*/
		public static function makeLineByPointAndVector( p0:Point, vp:Point ):Line{
			var a_:Number = vp.y;
			var b_:Number = -vp.x;
			var c_:Number = -a_*p0.x -b_*p0.y;
			return new Line( a_, b_, c_ );
		}
		
		/**
		* ２点を通る直線
		* @param	始点
		* @param	終点
		* @return	Line
		*/
		public static function makeLineBy2Point( p0:Point, p1:Point ):Line{
			return makeLineByPointAndVector( p0, p1.subtract( p0 ) );
		}
		
		/**
		* Fx
		* @param	x
		* @return	y
		*/
		public function fx( x:Number ):Number{
			return -(a*x+c)/b;
		}
		
		/**
		* Fy
		* @param	y
		* @return	x
		*/
		public function fy( y:Number ):Number{
			return -(b*y+c)/a;
		}
		
		/**
		* 直線の角度
		* @return radian
		*/
		public function get angle():Number{
			return Math.atan2( a, -b );
		}
		
		/**
		* ベクトル
		* @return vector
		*/
		public function get vector():Point{
			var v:Point = new Point( -b, a );
			v.normalize(1.0);
			return v;
		}
		
		/**
		* 直線と点の距離
		* @param	Point
		* @return	距離
		*/
		public function distanceToPoint( p:Point ):Number{
			return Math.abs( a*p.x + b*p.y + c )/Math.sqrt( a*a + b*b );
		}
		
		/**
		* 点から直線へのベクトル
		* @param	Point
		* @return	Vector
		*/
		public function vectorFromPoint( p:Point ):Point{
			//距離
			var v:Point = new Point( a, b );
			v.normalize( -( a*p.x + b*p.y + c )/v.length );
			return v;
		}
		
		/**
		* 直線と直線の交点
		* @param	Line
		* @return	Point
		*/
		public function intersectionTo( sl:Line ):Point{
			//外積
			var cross:Number = a*sl.b - sl.a*b;
			if( cross!=0 ){
				//交点
				return new Point( (b*sl.c - c*sl.b)/cross , (c*sl.a -sl.c*a)/cross );
			}else{
				//２つの直線は平行
				return null;
			}
		}
		
		public function toString():String{
			return "[Line ax+by+c=0 : a=" + a + ", b=" + b + ", c=" + c + " ]";
		}
	}
	
}
